"""
A Python client for interacting with the Center for Responsive Politics'
campaign finance data.

API docs: https://www.opensecrets.org/resources/create/api_doc.php
"""

import json
import os
import httplib2

try:
    import urllib.parse as urllib
except ImportError:
    import urllib as urllib


class CRPError(Exception):
    """ Exception for general CRP Client errors. """
    def __init__(self, message, response=None, url=None):
        super(CRPError, self).__init__(message)
        self.message = message
        self.response = response
        self.url = url


class Client(object):
    """
    The Client class handles retrieving and parsing responses from the
    OpenSecrets.org API.

    """

    BASE_URI = \
            'https://www.opensecrets.org/api/?method={method}&output=json&apikey={apikey}&{params}'

    def __init__(self, apikey=None, cache='.cache'):

        self.apikey = apikey
        self.http = httplib2.Http(cache)

    def fetch(self, method, **kwargs):
        """ Make the API request. """

        params = urllib.urlencode(kwargs)
        url = self.BASE_URI.format(method=method, apikey=self.apikey, params=params)
        headers = {'User-Agent' : 'Mozilla/5.0'}

        resp, content = self.http.request(url, headers=headers)
        content = json.loads(content)

        if not resp.get('status') == '200':
            raise CRPError(method, resp, url)

        return content['response']


class CandidatesClient(Client):
    """
    Retrieves and parses information pertaining to current Congressional
    legislators.
    """

    def get(self, id_code):
        """
            id_code may be either a candidate's specific CID, or a two letter
            state code, or a four character district code.
        """
        return self.fetch('getLegislators', id=id_code)['legislator']

    def pfd(self, cid, year=None):
        kwargs = {'cid' : cid}

        if year:
            kwargs['year'] = year

        return self.fetch('memPFDprofile', **kwargs)['member_profile']

    def summary(self, cid, cycle=None):
        kwargs = {'cid' : cid}

        if cycle:
            kwargs['cycle'] = cycle

        return self.fetch('candSummary', **kwargs)['summary']['@attributes']

    def contrib(self, cid, cycle=None):
        kwargs = {'cid' : cid}

        if cycle:
            kwargs['cycle'] = cycle

        return self.fetch('candContrib', **kwargs)['contributors']['contributor']

    def industries(self, cid, cycle=None):
        kwargs = {'cid' : cid}

        if cycle:
            kwargs['cycle'] = cycle

        return self.fetch('candIndustry', **kwargs)['industries']['industry']

    def contrib_by_ind(self, cid, industry, cycle=None):
        kwargs = {'cid' : cid, 'ind' : industry}

        if cycle:
            kwargs['cycle'] = cycle

        return self.fetch('candIndByInd', **kwargs)['candIndus']['@attributes']

    def sector(self, cid, cycle=None):
        kwargs = {'cid' : cid}

        if cycle:
            kwargs['cycle'] = cycle

        return self.fetch('candSector', **kwargs)['sectors']['sector']


class CommitteesClient(Client):
    """
    Retrieves and parses fundraising information pertaining to Congressional
    committees.
    """

    def cmte_by_ind(self, cmte, industry, congress=None):
        kwargs = {'cmte' : cmte, 'indus' : industry}

        if congress:
            kwargs['congno'] = congress

        return self.fetch('congCmteIndus', **kwargs)['committee']['member']


class OrganizationsClient(Client):
    """
    Retrieves and parses information pertaining to fundraising
    organizations.
    """

    def get(self, org_name):
        return self.fetch('getOrgs', org=org_name)['organization']

    def summary(self, org_id):
        return self.fetch('orgSummary', id=org_id)['organization']['@attributes']


class IndependentExpendituresClient(Client):
    """
    Retrieves and parses information regarding independent expenditure
    transactions.
    """

    def get(self):
        return self.fetch('independentExpend')['indexp']


class CRP(Client):
    """
    The public interface for the OpenSecrets.org API.

    Methods are namespaced by topic. Responses are returned as decoded JSON
    and trimmed for ease of use.

    An OpenSecrets.org API key is required, which can be passed as an argument
    when creating a new instance, or included as an environment variable.
    """

    def __init__(self, apikey=None, cache='.cache'):

        if apikey is None:
            apikey = os.environ.get('OPENSECRETS_API_KEY')

        super(CRP, self).__init__(apikey, cache)
        self.candidates = CandidatesClient(self.apikey, cache)
        self.committees = CommitteesClient(self.apikey, cache)
        self.orgs = OrganizationsClient(self.apikey, cache)
        self.indexp = IndependentExpendituresClient(self.apikey, cache)



try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

with open('README.rst') as f:
    README = f.read()

VERSION = '0.2.1'

setup(
    name = 'opensecrets-crpapi',
    version = VERSION,
    description = 'A Python client for the Center for Responsive Politics API at OpenSecrets.org.',
    long_description = README,
    url = 'https://github.com/robrem/opensecrets-crpapi',
    author = 'Rob Remington',
    author_email = 'rob@rob.codes',
    license = 'MIT',
    py_modules = ['crpapi'],
    install_requires = ['httplib2'],
    platforms = ['any'],
    classifiers = [
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Software Development :: Libraries :: Python Modules',

        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
    ],

    test_suite='test',
)


import json
import os
import unittest
import httplib2

from crpapi import CRP

API_KEY = os.environ['OPENSECRETS_API_KEY']

def close_connections(http):
    for k, conn in http.connections.items():
        conn.close()


class APITest(unittest.TestCase):

    def setUp(self):
        self.crp = CRP(API_KEY)
        self.http = httplib2.Http()

    def tearDown(self):
        close_connections(self.http)
        close_connections(self.crp.http)

    def check_response(self, result, url, parse=''):
        headers = {'User-Agent' : 'Mozilla/5.0'}
        response = self.http.request(url, headers=headers)[1]
        response = json.loads(response)

        if callable(parse):
            response = parse(response)

        self.assertEqual(result, response)


class CandidatesTest(APITest):

    def test_get(self):
        pelosi = self.crp.candidates.get('N00007360')
        url = \
            'https://www.opensecrets.org/api/?method=getLegislators&output=json&id=N00007360&apikey={0}' \
            .format(API_KEY)

        self.check_response(pelosi, url, lambda r: r['response']['legislator'])

    def test_pfd(self):
        pfd = self.crp.candidates.pfd('N00007360')
        url = \
            'https://www.opensecrets.org/api/?method=memPFDprofile&output=json&cid=N00007360&apikey={0}' \
            .format(API_KEY)

        self.check_response(pfd, url, lambda r: r['response']['member_profile'])

    def test_summary(self):
        summ = self.crp.candidates.summary('N00007360', '2014')
        url = \
            'https://www.opensecrets.org/api/?method=candSummary&output=json&cid=N00007360&cycle=2014&apikey={0}' \
            .format(API_KEY)

        self.check_response(summ, url, lambda r: r['response']['summary']['@attributes'])

    def test_contrib(self):
        contrib = self.crp.candidates.contrib('N00007360', '2014')
        url = \
            'https://www.opensecrets.org/api/?method=candContrib&output=json&cid=N00007360&cycle=2014&apikey={0}' \
            .format(API_KEY)

        self.check_response(contrib, url, lambda r: r['response']['contributors']['contributor'])

    def test_industries(self):
        ind = self.crp.candidates.industries('N00007360', '2014')
        url = \
            'https://www.opensecrets.org/api/?method=candIndustry&output=json&cid=N00007360&cycle=2014&apikey={0}' \
            .format(API_KEY)

        self.check_response(ind, url, lambda r: r['response']['industries']['industry'])

    def test_contrib_by_ind(self):
        ind = self.crp.candidates.contrib_by_ind('N00007360', 'F10', '2014')
        url = \
            'https://www.opensecrets.org/api/?method=candIndByInd&output=json&cid=N00007360&cycle=2014&ind=F10&apikey={0}' \
            .format(API_KEY)

        self.check_response(ind, url, lambda r: r['response']['candIndus']['@attributes'])

    def test_sector(self):
        sector = self.crp.candidates.sector('N00007360', '2014')
        url = \
            'https://www.opensecrets.org/api/?method=candSector&output=json&cid=N00007360&cycle=2014&apikey={0}' \
            .format(API_KEY)

        self.check_response(sector, url, lambda r: r['response']['sectors']['sector'])


class CommitteesTest(APITest):

    def test_cmte_by_ind(self):
        cmte = self.crp.committees.cmte_by_ind('HARM', 'F10', '113')
        url = \
            'https://www.opensecrets.org/api/?method=congCmteIndus&output=json&cmte=HARM&indus=F10&congno=113&apikey={0}' \
            .format(API_KEY)

        self.check_response(cmte, url, lambda r: r['response']['committee']['member'])


class OrganizationsTest(APITest):

    def test_get(self):
        org = self.crp.orgs.get('Goldman')
        url = \
            'https://www.opensecrets.org/api/?method=getOrgs&output=json&org=Goldman&apikey={0}' \
            .format(API_KEY)

        self.check_response(org, url, lambda r: r['response']['organization'])

    def test_summary(self):
        summ = self.crp.orgs.summary('D000000085')
        url = \
            'https://www.opensecrets.org/api/?method=orgSummary&output=json&id=D000000085&apikey={0}' \
            .format(API_KEY)

        self.check_response(summ, url, lambda r: r['response']['organization']['@attributes'])


class IndependentExpendituresTest(APITest):

    def test_get(self):
        exp = self.crp.indexp.get()
        url = \
            'https://www.opensecrets.org/api/?method=independentExpend&output=json&apikey={0}' \
            .format(API_KEY)

        self.check_response(exp, url, lambda r: r['response']['indexp'])


if __name__ == '__main__':
    unittest.main()
